package com.ruoyi.common.handler;


import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.MappedJdbcTypes;
import org.apache.ibatis.type.MappedTypes;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


@MappedJdbcTypes(value = { JdbcType.VARCHAR })
public class ConvertCommaSeparatedStrToListTypeHandler extends BaseTypeHandler<List<String>> {
    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, List<String> parameter, JdbcType jdbcType)
            throws SQLException {
        String d = parameter.stream().map(v -> String.valueOf(v)).collect(Collectors.joining(","));
        ps.setString(i, d);
    }

    @Override
    public List<String> getNullableResult(ResultSet rs, String columnName) throws SQLException {
        String values = rs.getString(columnName);
        return getResults(values);
    }

    @Override
    public List<String> getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        String values = rs.getString(columnIndex);
        return getResults(values);
    }

    @Override
    public List<String> getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        String values = cs.getString(columnIndex);
        return getResults(values);
    }

    private List<String> getResults(String values) {
        if (StringUtils.isNotBlank(values)) {
            String[] data = values.split(",");
            return new ArrayList<>(Arrays.asList(data));
        }
        return new ArrayList<>();
    }
//    public void setParameter(PreparedStatement preparedStatement, int i, List<String> strings, JdbcType jdbcType) throws SQLException {
//        // 遍历List类型的入参，拼装为String类型，使用Statement对象插入数据库
//        StringBuffer sb = new StringBuffer();
//        for (int j = 0; j < strings.size(); j++) {
//            if (j == strings.size() - 1) {
//                sb.append(strings.get(j));
//            } else {
//                sb.append(strings.get(j)).append(",");
//            }
//        }
//        preparedStatement.setString(i, sb.toString());
//    }

}
