package com.ruoyi.powerdistribution.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.powerdistribution.domain.PdmConstructor;
import com.ruoyi.powerdistribution.service.IPdmConstructorService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 配网工程队伍施工人员信息Controller
 * 
 * @author ruoyi
 * @date 2025-01-20
 */
@RestController
@RequestMapping("/powerdistribution/constructor")
public class PdmConstructorController extends BaseController
{
    @Autowired
    private IPdmConstructorService pdmConstructorService;

    /**
     * 查询配网工程队伍施工人员信息列表
     */
    @PreAuthorize("@ss.hasPermi('powerdistribution:constructor:list')")
    @GetMapping("/list")
    public TableDataInfo list(PdmConstructor pdmConstructor)
    {
        startPage();
        List<PdmConstructor> list = pdmConstructorService.selectPdmConstructorList(pdmConstructor);
        return getDataTable(list);
    }

    /**
     * 导出配网工程队伍施工人员信息列表
     */
    @PreAuthorize("@ss.hasPermi('powerdistribution:constructor:export')")
    @Log(title = "配网工程队伍施工人员信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PdmConstructor pdmConstructor)
    {
        List<PdmConstructor> list = pdmConstructorService.selectPdmConstructorList(pdmConstructor);
        ExcelUtil<PdmConstructor> util = new ExcelUtil<PdmConstructor>(PdmConstructor.class);
        util.exportExcel(response, list, "配网工程队伍施工人员信息数据");
    }

    /**
     * 获取配网工程队伍施工人员信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('powerdistribution:constructor:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(pdmConstructorService.selectPdmConstructorById(id));
    }

    /**
     * 新增配网工程队伍施工人员信息
     */
    @Log(title = "配网工程队伍施工人员信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PdmConstructor pdmConstructor)
    {
        return toAjax(pdmConstructorService.insertPdmConstructor(pdmConstructor));
    }

    /**
     * 修改配网工程队伍施工人员信息
     */
    @Log(title = "配网工程队伍施工人员信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PdmConstructor pdmConstructor)
    {
        return toAjax(pdmConstructorService.updatePdmConstructor(pdmConstructor));
    }

    /**
     * 删除配网工程队伍施工人员信息
     */

    @Log(title = "配网工程队伍施工人员信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(pdmConstructorService.deletePdmConstructorByIds(ids));
    }
}
