package com.ruoyi.powerdistribution.controller;

import java.math.BigDecimal;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.github.pagehelper.PageInfo;
import com.ruoyi.common.utils.bean.BeanListUtils;
import com.ruoyi.powerdistribution.domain.PdmScoreRankTeam;
import com.ruoyi.powerdistribution.domain.vo.*;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.powerdistribution.domain.PdmScoreRankManager;
import com.ruoyi.powerdistribution.service.IPdmScoreRankManagerService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 负责人积分排名Controller
 *
 * @author ruoyi
 * @date 2024-12-05
 */
@RestController
@RequestMapping("/power/managerRank")
public class PdmScoreRankManagerController extends BaseController
{
    @Autowired
    private IPdmScoreRankManagerService pdmScoreRankManagerService;

    /**
     * 查询负责人积分排名列表
     */
    @ApiOperation(value = "查询负责人积分排名列表")
    @GetMapping("/list")
    public TableDataInfo list(PdmScoreRankManager pdmScoreRankManager)
    {
        startPage();
        List<PdmScoreRankManager> list = pdmScoreRankManagerService.selectScoreRankManagerList(pdmScoreRankManager);
        return getDataTable(list);
    }

    /**
     * 导出负责人积分排名列表
     */
    @ApiOperation(value = "导出负责人积分排名列表")
    @Log(title = "负责人积分排名", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PdmScoreRankManager pdmScoreRankManager)
    {
        List<PdmScoreRankManager> list = pdmScoreRankManagerService.selectScoreRankManagerList(pdmScoreRankManager);
        if ("2".equals(pdmScoreRankManager.getTeamType()) && StringUtils.isNotEmpty(pdmScoreRankManager.getReportYear())) {
            //外部-年
            List<PdmScoreRankManagerYearOut> newList = BeanListUtils.copyListProperties(list,PdmScoreRankManagerYearOut::new);
            ExcelUtil<PdmScoreRankManagerYearOut> util = new ExcelUtil<>(PdmScoreRankManagerYearOut.class);
            util.exportExcel(response, newList, "负责人积分排名数据");
        }else if ("2".equals(pdmScoreRankManager.getTeamType()) && StringUtils.isEmpty(pdmScoreRankManager.getReportYear())) {
            //外部-月
            List<PdmScoreRankManagerOut> newList = BeanListUtils.copyListProperties(list,PdmScoreRankManagerOut::new);
            ExcelUtil<PdmScoreRankManagerOut> util = new ExcelUtil<>(PdmScoreRankManagerOut.class);
            util.exportExcel(response, newList, "负责人积分排名数据");
        }else if("1".equals(pdmScoreRankManager.getTeamType()) && StringUtils.isNotEmpty(pdmScoreRankManager.getReportYear())){
            //内部-年
            List<PdmScoreRankManagerYear> newList = BeanListUtils.copyListProperties(list,PdmScoreRankManagerYear::new);
            ExcelUtil<PdmScoreRankManagerYear> util = new ExcelUtil<>(PdmScoreRankManagerYear.class);
            util.exportExcel(response, newList, "负责人积分排名数据");
        } else{
            //内部-月
            ExcelUtil<PdmScoreRankManager> util = new ExcelUtil<>(PdmScoreRankManager.class);
            util.exportExcel(response, list, "负责人积分排名数据");
        }

    }
    /**
     * 获取负责人积分排名详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(pdmScoreRankManagerService.selectPdmScoreRankManagerById(id));
    }

    /**
     * 新增负责人积分排名
     */
    @Log(title = "负责人积分排名", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PdmScoreRankManager pdmScoreRankManager)
    {
        return toAjax(pdmScoreRankManagerService.insertPdmScoreRankManager(pdmScoreRankManager));
    }

    /**
     * 修改负责人积分排名
     */
    @Log(title = "负责人积分排名", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PdmScoreRankManager pdmScoreRankManager)
    {
        return toAjax(pdmScoreRankManagerService.updatePdmScoreRankManager(pdmScoreRankManager));
    }

    /**
     * 删除负责人积分排名
     */
    @Log(title = "负责人积分排名", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(pdmScoreRankManagerService.deletePdmScoreRankManagerByIds(ids));
    }
}
